@@ -1,10 +1,11 @@
 from .node import Node
+from . import errors
 
 
 class Person(Node):  # is-a Node only because that holds path,
                      # docname, and userdata
-    def __init__(self, title, path, docname, userdata, firstname,
-                 lastname):
+    def __init__(self, title, path, docname, userdata, 
+                 firstname, lastname):
         super().__init__(
             title=title, 
             path=path, 
@@ -12,8 +13,11 @@ class Person(Node):  # is-a Node only because that holds path,
             userdata=userdata,
             dependencies=[], 
             )
+
+        if not firstname:
+            raise OpenTrainingError('firstname must be set')
+        if not lastname:
+            raise OpenTrainingError('lastname must be set')
+
         self.firstname = firstname
         self.lastname = lastname
-
-    def __str__(self):
-        return f'Person:{str(super())}, firstname:{self.firstname}, lastname:{self.lastname}'
